
#include <conio.h>
#include "svga.h"

/*

 Identifying the Tseng SVGA Card 


Tseng Labs have produced two SVGA Chips, the ET3000 and the ET4000.

The Tseng SVGA chips can be identified by attempting to change the
Miscellaneous register as follows:

          Index : 06h at port 3C0h
          Read/write data from port 3C1h
          Ŀ
	   7  6  5  4  3  2  1  0 
	  
		    
                     High

Output the value 6 to port 3C0h and read a byte from port 3C1h. Modify
the high field in this byte (eg new byte = byte XOR 30h) and write this
new byte to port 3C1h. Read the byte from port 3C1h and see if the byte
was successfully modified, if it was then a Tseng chip is present. Having
done this, write the original byte back to port 3C1h to leave the graphics
adapter in it's original state.


 Identifying which Tseng Card is Present 


The ET4000 can be distinguished from the ET3000 by attempting to change the
ET4000 Extended Start Address register as follows:

	  Index : 33h at port 3D4h
          Read/write data from port 3D5h
          Ŀ
	   7  6  5  4  3  2  1  0 
	  
			       
			     CAD     DAD

The same technique is used as was used to identify the presence of a
Tseng chip, both fields should be modified, written, tested for a successful
write and then restored to their original values. If the change was
successful an ET4000 chip is present, otherwise an ET3000 chip is.
*/

static int ET_probe (int Port, unsigned char Value, unsigned char Xor)
{
  unsigned char a,b;
  int result;

  outp (Port, Value);
  a = inp (Port+1);
  b = a ^ Xor;
  outp (Port+1, b);
  result = (inp (Port+1) == b);
  outp (Port+1, a);
  return result;
}

int ET_detect (void)
{
  int result;
  int et, et4000;

  et = ET_probe (0x3C0, 0x06, 0x20);
  et4000 = ET_probe (0x3D4, 0x33, 0x0F);

  if (et4000)
    result = XVGA_ET4000;
  else
    if (et)
      result = XVGA_ET3000;
    else
      result = XVGA_VGA;

  return result;
}
