
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <dos.h>
#include <direct.h>

#include "cgdk.h"
#include "file.h"

long F_length (const char *str)
{
  FILE *file;
  long len;

  file = fopen (str, "rb");
  if (file == NULL)
  {
    return -1;
  }

  if (fseek (file, 0, SEEK_END) != 0)
  {
    fclose (file);
    return -2;
  }

  len = ftell (file);
  fclose (file);
  if (len == -1)
  {
    return -3;
  }

  return len;
}


int F_write (const char *str, void *buf, unsigned size)
{
  FILE *file;

  if (file = fopen (str, "wb"), file == NULL)
  {
    return -1;
  }

  if (fwrite (buf, 1, size, file) != size)
  {
    fclose (file);
    return -2;
  }
  fclose (file);
  return 0;
}

int F_append (const char *str, void *buf, unsigned size)
{
  FILE *file;

  if ((file = fopen (str, "a+b")) == NULL)
  {
    return -1;
  }

  if (fwrite (buf, 1, size, file) != size)
  {
    fclose (file);
    return -2;
  }
  fclose (file);
  return 0;
}


int F_read (const char *str, void *buf, unsigned size, unsigned position)
{
  FILE *file;
  unsigned result;

  file = fopen (str, "rb");
  if (file == NULL)
  {
    return -1;
  }

  if (fseek (file, position, SEEK_SET) != 0)
  {
    return -2;
  }

  result = fread (buf, 1, size, file);
  fclose (file);

  return result;
}

void *F_load (const char *str)
{
  FILE *file;
  long len;
  void *buf;

  file = fopen (str, "rb");
  if (file == NULL)
  {
    return NULL;
  }

  if (fseek (file, 0, SEEK_END) != 0)
  {
    fclose (file);
    return NULL;
  }

  len = ftell (file);
  if (len == -1)
  {
    fclose (file);
    return NULL;
  }

  if (fseek (file, 0, SEEK_SET) != 0)
  {
    fclose (file);
    return NULL;
  }

  buf = _alloc (len);
  if (buf == NULL)
  {
    fclose (file);
    return NULL;
  }

  if (fread (buf, 1, len, file) != len)
  {
    fclose (file);
    _free (buf);
    return NULL;
  }

  return buf;
}

int F_exist (const char *str)
{
  struct find_t fblock;

  return (_dos_findfirst (str, 0, &fblock)==0);
}





static int F_findAgain = 1;
static struct find_t F_ffblk;

static char F_drive[_MAX_DRIVE];
static char F_dir[_MAX_DIR];
static char F_path[_MAX_PATH];

char *F_findNext (const char *wildName)
{
  char *ptr;

  char F_name[_MAX_FNAME];
  char F_ext[_MAX_EXT];

  if (F_findAgain)
  {
    _splitpath (wildName, F_drive, F_dir, F_name, F_ext);
    if (!_dos_findfirst (wildName, 0, &F_ffblk))
    {
      _splitpath (F_ffblk.name, NULL, NULL, F_name, F_ext);
      _makepath (F_path, F_drive, F_dir, F_name, F_ext);
      F_findAgain = 0;
      return F_path;
    }
    else
    {
      return NULL;
    }
  }
  else
  {
    if (!_dos_findnext (&F_ffblk))
    {
//      _splitpath (F_ffblk.name,NULL,NULL,F_name,F_ext);
                _makepath(F_path,F_drive,F_dir,F_ffblk.name,NULL);
      return F_path;
    }
    else
    {
      return NULL;
    }
  }
}

void F_findReset (void)
{
  F_findAgain = 1;
}

int F_fcmp(const void *a,const void *b)
{
        return(stricmp(*(char **)a,*(char **)b));
}


TF_list *F_listNew (const char *wildName)
{
  char *F_path;
  TF_list *result;
  int size;
  int i;

  result = (TF_list *) _alloc (sizeof (TF_list));
  if (result)
  {
    size = sizeof (char *);
    result->names = (char * *) _alloc (size);
    if (result == NULL)
    {
      return NULL;
    }
    i = 0;
    F_findReset ();
    while (F_path = F_findNext (wildName), F_path != NULL)
    {
      size += sizeof (char *);
      result->names = (char * *) _realloc (result->names, size,NULL);
      if (result == NULL)
      {
        return NULL;
      }
      result->names [i] = (char *) _alloc (strlen (F_path)+1);
      if (result->names [i] == NULL)
      {
        return NULL;
      }
      else
      {
        strcpy (result->names [i], F_path);
      }
      i++;
    }
    result->cur = -1;
    result->max = i;
    qsort(result->names, result->max, sizeof(char *),F_fcmp);

    return result;
  }
  else
  {
    return NULL;
  }
}

void F_listDelete (TF_list *list)
{
  int i;

  if (list != NULL)
  {
    for (i = 0; i < list->max; i++)
    {
      _free (list->names [i]);
    }
    _free (list);
  }
}

char *F_listNext (TF_list *list)
{
  if (list != NULL)
  {
    if (list->cur < list->max-1)
    {
      list->cur++;
      return list->names [list->cur];
    }
    else
    {
      return NULL;
    }
  }
  else
  {
    return NULL;
  }
}

char *F_listPrev (TF_list *list)
{
  if (list != NULL)
  {
    if (list->cur > 0)
    {
      list->cur--;
      return list->names [list->cur];
    }
    else
    {
      return NULL;
    }
  }
  else
  {
    return NULL;
  }
}

void F_getExecPath (char *fullPath, const char *fileName)
{
  char drive[_MAX_DRIVE];
  char dir[_MAX_DIR];
  char path[_MAX_PATH];
  char name [_MAX_FNAME];
  char ext [_MAX_EXT];

  _splitpath (_argv [0], drive, dir, NULL, NULL);
  _splitpath (fileName, NULL, NULL, name, ext);
  _makepath (fullPath, drive, dir, name, ext);
  strupr (fullPath);
}


void F_getFullPath (char *path, const char *fileName)
{
  unsigned disk;

  getcwd (path, 255);
  strcat (path, "\\");
  strcat (path, fileName);
  strupr (path);
}
