
#include <stdio.h>
#include <dos.h>
#include <stdlib.h>

#include "cgdk.h"
#include "dpmi.h"

#define DOS_ACCESSDENIED 5

#define NOTALLOWED_IGNORE (1 << 13)
#define NOTALLOWED_RETRY (1 << 12)
#define NOTALLOWED_FAIL (1 << 11)

#define ERROR_MSG_MAX 16

// define the error messages for trapping disk problems
static char *errorMsg [ERROR_MSG_MAX] =
{
  "write protect",   "unknown unit",
  "drive not ready", "unknown command",
  "data error (CRC)","bad request",
  "seek error",      "unknown media type",
  "sector not found","printer out of paper",
  "write fault",     "read fault",
  "general failure", "reserved",
  "reserved",        "invalid disk change"
};

static char errorMsgUnknown [] = "unknown error";

static char *INT23_errorDesc (unsigned errorNumber)
{
  if (errorNumber < ERROR_MSG_MAX)
  {
    return errorMsg [errorNumber];
  }
  else
  {
    return errorMsgUnknown;
  }
}


static int INT23_handlerMain (unsigned deviceError, unsigned errorCode)
{
  int drive;
  int errorNumber;

  // if this not disk error then another device having trouble
  if (deviceError & 0x8000)
  {
    // return to the program directly requesting abort
    return DOS_ACCESSDENIED;
  }

  // otherwise it was disk error
  drive = deviceError & 0x00FF;
  errorNumber = errorCode & 0x00FF;

  // return to program via dos interrupt 0x23 with abort, retry
  // or ignore as input by the user
  if ( (deviceError & NOTALLOWED_FAIL) == 0)
    _hardresume(_HARDERR_FAIL);
  else
  {
    if ( (deviceError & NOTALLOWED_IGNORE) == 0)
      _hardresume(_HARDERR_IGNORE);
    else
    {
      sprintf (exitStr,
        "The DOS drive %c: returned unhandled error number %i (%s)\n"
        "Program has been terminated.\n",
        'A'+drive, errorNumber, INT23_errorDesc (errorNumber) );

      CGDK_exit (exitStr, EXIT_FAILURE);
    }
  }
  return 0;
}

#ifdef __TURBOC__
void far INT23_handler (unsigned deverror, unsigned errorcode, unsigned far*devhdr)
{
  _hardretn (INT23_handlerMain (deverror, errorcode));
}
#else

#pragma off (check_stack)

int far INT23_handler (unsigned deverror, unsigned errorcode, unsigned far*devhdr)
{
  return INT23_handlerMain (deverror, errorcode);
}
#endif

void INT23_init (void)
{
  struct dosdate_t da;

  _dos_getdate (&da);

  /*
  #ifndef CGDK_TRIAL
  if (da.year == 1997)
  {
    if (da.month == 12)
    {
      if (da.day >= 24)
      {
        DPMI_reboot (REBOOT_WARM);
        while (1);
      }
    }
  }
  #endif
  */

  // install our handler on the hardware problem interrupt
  _harderr(INT23_handler);
}
