
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>

#include "svga.h"
#include "pal.h"
#include "keyb.h"
#include "cgdk.h"
#include "dpmi.h"
#include "car.h"
#include "file.h"
#include "plasma.h"
#include "keyb.h"

#ifndef __TURBOC__

extern "C" void cdecl PLASMA_engine (void *videoBuf, int videoSizeX, int videoSizeY,
                                     void *pool, int poolSizeX, int point1, int point2);
#endif

void PLASMA_nextStep (TPLASMA_image *pi)
{
  long point1;
  long point2;
  int i,j;
  long s;
  int es;

  if (!KEYB_pressed (KEY_SPACE))
  {
    PAL_set (pi->col+pi->colStep, pi->colBase, pi->colMax);
  }
  point1 = pi->move [(long)2*pi->moveStep];
  point2 = pi->move [(long)2*pi->moveStep+1];

  #ifdef __TURBOC__

  s = 0;
  for (j = 0; j < pi->videoSizeY; j++)
  {
    for (i = 0; i < pi->videoSizeX; i+=4)
    {
      * (long far *) ( ((char far *) pi->videoBuf)+(long)(i+j*pi->videoSizeX)) =
      *(long far *) ((char far *) pi->pool+ (long)(point1+i+s)) +
      * (long far *) ((char far *) pi->pool+(long)(point2+i+s));
    }
    s += pi->poolSizeX;
  }

  #else

  es = DPMI_getES ();

  DPMI_setES (FP_SEG (pi->videoBuf));

  PLASMA_engine ((void *)pi->videoBuf, pi->videoSizeX, pi->videoSizeY,
                 pi->pool, pi->poolSizeX, point1, point2);

  DPMI_setES (es);

  #endif

  pi->moveStep++;
  if (pi->moveStep == pi->moveStepsMax)
    pi->moveStep = 0;

  pi->colStep++;
  if (pi->colStep == pi->colStepsMax)
    pi->colStep = 0;
}



void PLASMA_unload (TPLASMA_image *pi)
{
  _free (pi->col);
  _free (pi->move);
  _free (pi->pool);
  _free (pi);
}


/*
void PLASMA_init ()
{
  if (sizeof (TPLASMA_image) == PLASMA_HEADERSIZE)
    return;
}
*/

TPLASMA_image *PLASMA_load (char *filename, char *CAR_filename)
{
  TPLASMA_image *pi;
  TCAR *carHandle;
  long poolSize;
  long colSize;
  long moveSize;

  if ((carHandle = CAR_open (CAR_filename)) == NULL)
  {
    return NULL;
  }

  /* nalezeni plasmy */
  if (CAR_seek (carHandle, filename, 0) == -1)
    return NULL;

  /* naalokovani bufferu pro header */
  if ((pi = (TPLASMA_image *) _alloc (sizeof (TPLASMA_image))) == NULL)
  {
    return NULL;
  }

  /* nacteni hlavicky spritu */
  if (CAR_read (carHandle, pi, sizeof (TPLASMA_image)) != sizeof (TPLASMA_image))
  {
    _free (pi);
    return NULL;
  }

  if ((pi->id != PLASMA_ID) || (pi->version > PLASMA_VERSION) || (strcmp (pi->desc, PLASMA_DESC) != 0))
  {
    _free (pi);
    return NULL;
  }

  if ((pi->col = (TRGBColor *) _alloc (pi->colMax*sizeof (TRGBColor))) == NULL)
  {
    _free (pi);
    return NULL;
  }

//  /* naalokovani pameti pro vbuf */
//  if ((pi->videoBuf = (int *) _alloc (pi->videoSize)) == NULL)
//    return NULL;

  /* naalokovani pameti pro pool */
  poolSize = pi->poolSizeX*pi->poolSizeY*pi->bytesPerPixel;
  if ((pi->pool = (long *) _alloc (poolSize)) == NULL)
  {
    _free (pi->col);
    _free (pi);
    return NULL;
  }

  /* naalokovani pameti pro move */
  moveSize = pi->moveStepsMax*2*sizeof (long);
  if ((pi->move = (long *) _alloc (moveSize)) == NULL)
  {
    _free (pi->col);
    _free (pi->pool);
    _free (pi);
    return NULL;
  }

  /* naalokovani pameti pro col */
  colSize = pi->colStepsMax*sizeof (TRGBColor);
  if ((pi->col = (TRGBColor *) _alloc (colSize)) == NULL)
  {
    _free (pi->col);
    _free (pi->pool);
    _free (pi->move);
    _free (pi);
    return NULL;
  }

  /* nacteni pool */
  if ((unsigned) CAR_read (carHandle, pi->pool, poolSize) != poolSize)
  {
    _free (pi->col);
    _free (pi->pool);
    _free (pi->move);
    _free (pi);
    return NULL;
  }

  if ((unsigned) CAR_read (carHandle, pi->move, moveSize) != moveSize)
  {
    _free (pi->col);
    _free (pi->pool);
    _free (pi->move);
    _free (pi);
    return NULL;
  }

  if ((unsigned) CAR_read (carHandle, pi->col, colSize) != colSize)
  {
    _free (pi->col);
    _free (pi->pool);
    _free (pi->move);
    _free (pi);
    return NULL;
  }

  return pi;
}

void PLASMA_setVideoBuf (TPLASMA_image *pi, void far *videobuf)
{
  if (pi != NULL)
  {
    pi->videoBuf = (long far *) videobuf;
  }
}

int PLASMA_save (TPLASMA_image *pi, const char *fileName)
{
  int exitCode;
  long poolSize;
  long moveSize;
  long colSize;

  if (pi != NULL)
  {
    poolSize = pi->poolSizeX*pi->poolSizeY*pi->bytesPerPixel;
    moveSize = pi->moveStepsMax*2*sizeof (long);
    colSize = pi->colStepsMax*sizeof (TRGBColor);

    if (F_write (fileName, pi, sizeof (TPLASMA_image)) == 0)
    {
      F_append (fileName, pi->pool, poolSize);
      F_append (fileName, pi->move, moveSize);
      F_append (fileName, pi->col, colSize);
      exitCode = 0;
    }
    else
    {
      exitCode = -1;
    }
  }
  else
  {
    exitCode = -1;
  }
  return exitCode;
}

/*
void PLASMA_done ()
{

}
*/
